// Phatsoft JoyMouse - A lightweight utility to control the mouse cursor with a joystick
// Copyright (C) 2002  Luca Martinetti
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "stdafx.h"
#include "joymouse.h"
#include <stdlib.h>
#include "SendItem.h"

#define URL_DONATE "http://order.kagi.com/cgi-bin/store.cgi?storeID=6CXJT&&"
//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: Entry point for the application.
//-----------------------------------------------------------------------------
int APIENTRY WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, int )
{
		g_hInst = hInst;

		// Register Class

		WNDCLASSEX wcx;
		ZeroMemory(&wcx,sizeof(WNDCLASSEX));
		
		wcx.cbSize = sizeof(wcx);          
		wcx.lpszClassName = "JoyMouseWnd";
		wcx.lpfnWndProc = MainWndProc;
		wcx.hInstance = g_hInst;
		
		ATOM aClass = RegisterClassEx(&wcx);
		_ASSERTE(aClass);

		// Create MainWindow
		g_hwndMain = CreateWindowEx(0,(LPCTSTR)MAKELONG(aClass,0),"Phatsoft JoyMouse!",0,0,0,0,0,(HWND) NULL,(HMENU) NULL,hInst,NULL);
		_ASSERTE(g_hwndMain);
		
		// Create Tray Icon
		g_TrayIcon = new ZTray(g_hwndMain, 0, WM_TRAY);
		
		g_TrayIcon->Show("Phatsoft JoyMouse",hInst , IDI_MOUSED);
		g_TrayIcon->ChangeTip("Joystick not connected!");

		if (!g_EnableAtLaunch) {
			g_TrayIcon->ChangeIcon(g_hInst, IDI_MOUSED);
			g_bEnabled = false;
		}
		
		// Update Loop Thread
		DWORD dwThreadID;
		HANDLE hThread = CreateThread(NULL,0,UpdateLoop,NULL,0,&dwThreadID);
		_ASSERTE(hThread);
		SetThreadPriority(hThread,THREAD_PRIORITY_TIME_CRITICAL);

		// Message Loop
		BOOL bRet;
		MSG msg;
		
		while( (bRet = GetMessage( &msg, NULL, 0, 0 )) != 0)
    { 
        if (bRet == -1)
        {
            _ASSERTE(false);
        }
        else
        {
            TranslateMessage(&msg); 
            DispatchMessage(&msg); 
        }
    }
		
		// Delete Tray icon
		if (g_TrayIcon) delete g_TrayIcon;

		// Stop Update Loop
		g_bUpdateLoop = false;
		
		// Return exit code
		return msg.wParam; 
}


//-----------------------------------------------------------------------------
// Name: MainWndProc
// Desc: Handles dialog messages
//-----------------------------------------------------------------------------
LRESULT CALLBACK MainWndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    switch( msg ) 
    {
				case WM_CREATE:
					{ 
						LoadSettings();
						return TRUE;
					}
					break;

					case WM_COMMAND:
					{
						
						switch(LOWORD(wParam)) 
						{ 
								case ID_TRAY_OPTIONS:
								{
									if(!g_hwndOptions)
									{
										ShowOptions();
									}
									else
									{
										SetForegroundWindow(g_hwndOptions);
									}
									return TRUE;
								}
								break;

								case ID_TRAY_ABOUT: 
								{
									ShowAboutBox();
									return TRUE;
								}
								break;
								
								case ID_TRAY_EXIT:
								{
									PostQuitMessage(0);
									return TRUE;
								}
								break;
								
								case ID_TRAY_ENABLED:
								{
									Enable(!g_bEnabled);
									return TRUE;
								}
								break;

								default:
								{
									return FALSE;
								}
						} 

					}
					break;

				case WM_TRAY:
					{
						_ASSERTE(wParam == 0);
						switch(lParam)
						{
							case WM_LBUTTONUP:
							{
									Enable(!g_bEnabled);
									return TRUE;
							}
							break;
							
							case WM_RBUTTONUP:
							{
								ShowContextMenu();
								return TRUE;
							}
							break;
						}
					}
				
					default:
					{
						return DefWindowProc(hWnd,msg,wParam,lParam);
					}
					break;
    }
}

BOOL RefreshState()
{
	
	int i;

	// Get Joystick info
	JOYINFOEX ji;
	ji.dwSize = sizeof(ji);
	ji.dwFlags = JOY_RETURNX | JOY_RETURNY | JOY_RETURNBUTTONS | JOY_USEDEADZONE;
	if ( joyGetPosEx(JOYSTICKID1,&ji) != JOYERR_NOERROR) return false;

	if (g_bEnabled) {	
		long dx = 0;
		long dy = 0;
		long mult = 0;
		
		DWORD dwXmid = ((g_jc.wXmax - g_jc.wXmin)/2)+1;
		DWORD dwYmid = ((g_jc.wXmax - g_jc.wXmin)/2)+1;

		if (ji.dwXpos < (dwXmid - g_Step)) dx = -1;
		if (ji.dwXpos > (dwXmid + g_Step)) dx = 1;
	
		if (ji.dwYpos < (dwYmid - g_Step)) dy = -1;
		if (ji.dwYpos > (dwYmid + g_Step)) dy = 1;

		if (g_bAccel)
			mult = g_Speed2;
		else
			mult = g_Speed;
	
		// Update cursor pos
		POINT p;
		GetCursorPos(&p);
		p.x += (dx * mult);
		p.y += (dy * mult);
		SetCursorPos(p.x,p.y);
	}
	
	// buttons actions
	for(i = 0; i < BUTTON_COUNT; ++i) 
	{
		if(ji.dwButtons & g_Button_Data[i*4]) 
		{
			ButtonAction(g_Button_Data[i*4+1], true);
			SendDlgItemMessage(g_hwndOptions, g_Button_Data[i*4+3], BM_SETCHECK, TRUE, 0);
		} 
		else 
		{
			ButtonAction(g_Button_Data[i*4+1], false);
			SendDlgItemMessage(g_hwndOptions, g_Button_Data[i*4+3], BM_SETCHECK, FALSE, 0);
		}
	}	

	return TRUE;
}


VOID ShowContextMenu() 
{ 
    HMENU hMenu;            // top-level menu 
    HMENU hSubMenu;  // shortcut menu 
 
    // Load the menu resource. 
 
		hMenu = LoadMenu(g_hInst,MAKEINTRESOURCE(IDR_MENU_TRAY));
		_ASSERTE(hMenu);
		
    // TrackPopupMenu cannot display the menu bar so get 
    // a handle to the first shortcut menu. 
 
    hSubMenu = GetSubMenu(hMenu, 0);
		_ASSERTE(hSubMenu);
		
		// Set default item
		SetMenuDefaultItem(hSubMenu,ID_TRAY_ENABLED,FALSE);

		// Check enabled if needed
		MENUITEMINFO mii;
		mii.cbSize = sizeof(MENUITEMINFO);
		mii.fMask = MIIM_STATE;
		mii.fState = (g_bEnabled) ? MFS_CHECKED : MFS_UNCHECKED;

		SetMenuItemInfo(hMenu,ID_TRAY_ENABLED,FALSE,&mii);

    // Display the shortcut menu
		POINT p;
		GetCursorPos(&p);
    TrackPopupMenu(hSubMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, p.x, p.y, 0, g_hwndMain, NULL); 
 
    // Destroy the menu. 
    DestroyMenu(hMenu); 
} 

INT_PTR CALLBACK OptionsDlgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
		switch(msg)
    {
				case WM_INITDIALOG:
				{
					// Store Handle in global
					g_hwndOptions= hWnd;

					// Speed
					HWND hwndSpeed = GetDlgItem(hWnd,IDC_SLIDER_SPEED);
					_ASSERTE(hwndSpeed);
					SendMessage(hwndSpeed,TBM_SETRANGE,(WPARAM)(BOOL)FALSE,(LPARAM) MAKELONG(1, 10));
					SendMessage(hwndSpeed,TBM_SETPOS,(WPARAM)TRUE,(LPARAM)g_Speed);
					
					// Faster speed
					HWND hwndSpeed2 = GetDlgItem(hWnd,IDC_SLIDER_SPEED_2);
					_ASSERTE(hwndSpeed2);
					SendMessage(hwndSpeed2,TBM_SETRANGE,(WPARAM)(BOOL)FALSE,(LPARAM) MAKELONG(1, 15));
					SendMessage(hwndSpeed2,TBM_SETPOS,(WPARAM)TRUE,(LPARAM)g_Speed2);

					// Threshold
					HWND hwndStep = GetDlgItem(hWnd,IDC_SLIDER_STEP);
					_ASSERTE(hwndStep);
					SendMessage(hwndStep,TBM_SETRANGE,(WPARAM)(BOOL)FALSE,(LPARAM) MAKELONG(1, (UINT)(g_jc.wXmax/2)));
					SendMessage(hwndStep,TBM_SETPOS,(WPARAM)TRUE,(LPARAM)g_Step);
					
					// Pool range
					HWND hwndPool = GetDlgItem(hWnd,IDC_SLIDER_POOL);
					_ASSERTE(hwndStep);
					SendMessage(hwndPool,TBM_SETRANGE,(WPARAM)(BOOL)FALSE,(LPARAM) MAKELONG(20, 1000));
					SendMessage(hwndPool,TBM_SETPOS,(WPARAM)TRUE,(LPARAM)g_Pool);

					// Enable at start
					HWND hwndEnable = GetDlgItem(hWnd, IDC_CHECK_ENABLED);
					_ASSERTE(hwndEnable);
					SendMessage(hwndEnable,BM_SETCHECK,(WPARAM)g_EnableAtLaunch, 0);

					// Various buttons
					PopulateCombos();

					return TRUE;
				}
				break;

				case WM_CLOSE:
				{
					LoadSettings();
					g_hwndOptions = NULL;
					EndDialog(hWnd,0);
					return TRUE;
				}
				break;

				case WM_COMMAND:
				{
					switch(LOWORD(wParam))
					{
						case IDOK:
						{
							if (!SaveSettings())
							{
								MessageBox(g_hwndMain,"Can't save registry settings","Phatsoft JoyMouse",MB_ICONERROR | MB_SETFOREGROUND);
							}
							g_hwndOptions = NULL;
							EndDialog(hWnd,0);

							return TRUE;
						}
						break;
						
						case IDC_STATIC_MOUSE:
						{
							if ( HIWORD(wParam) == STN_CLICKED)	
							{ 
								ShowAboutBox();
								return TRUE;
							}
							return FALSE;
						}
						break;
						case IDC_CHECK_ENABLED:
						{
							if (SendDlgItemMessage(g_hwndOptions, IDC_CHECK_ENABLED, BM_GETCHECK, 0, 0) == BST_CHECKED) {
								g_EnableAtLaunch = true;
							} else {
								g_EnableAtLaunch = false;
							}
						}
						break;

						case IDC_STATIC_KAGI:
						{
							GotoURL(URL_DONATE,SW_NORMAL);
							return TRUE;
						}
						break;
							
						default:
						{
							int i;
							for (i = 0; i < BUTTON_COUNT; ++i) {
								if ((LOWORD(wParam) == g_Button_Data[i*4+2]) &&
										(HIWORD(wParam) == CBN_SELCHANGE)) {
									g_Button_Data[i*4+1] = SendMessage((HWND) lParam, CB_GETCURSEL,0,0);
									UnsetAction(g_Button_Data[i*4+1], i);
									return TRUE;
								}
							}
							return FALSE;
						}
						break;

					}
					break;
				}

				case WM_HSCROLL:
				{
					switch(LOWORD(wParam))
					{
						case TB_ENDTRACK:
						{
							_ASSERTE((HWND)lParam);
							int idSlider = GetDlgCtrlID((HWND)lParam);
							
							switch(idSlider)
							{
								case IDC_SLIDER_SPEED:
								{
									g_Speed = SendMessage((HWND)lParam,TBM_GETPOS,0,0);
									return TRUE;
								}
								break;

								case IDC_SLIDER_SPEED_2:
								{
									g_Speed2 = SendMessage((HWND)lParam,TBM_GETPOS,0,0);
									return TRUE;
								}
								break;
								
								case IDC_SLIDER_STEP:
								{
									g_Step = SendMessage((HWND)lParam,TBM_GETPOS,0,0);
									return TRUE;
								}
								break;
								
								case IDC_SLIDER_POOL:
								{
									g_Pool = SendMessage((HWND)lParam,TBM_GETPOS,0,0);
									return TRUE;
								}
								break;

								default: // Other controll
								{
									return FALSE;
								}

							}
						}
						break;
						
						default: // Other scroll event
						{
							return FALSE;
						}
						break;
					}
				}
				break;

				default: // Other msg
				{
					return FALSE;
				}
				break;
    }
}

BOOL SaveSettings()
{
	CRegistry reg;
	char temp[20];
	int i;
	
	BOOL bRet = reg.SaveKey(g_pszKey,"Speed",g_Speed);
	bRet = bRet && reg.SaveKey(g_pszKey,"Speed2",g_Speed2);
	bRet = bRet && reg.SaveKey(g_pszKey,"Pool",g_Pool);
	bRet = bRet && reg.SaveKey(g_pszKey,"Step",g_Step);
	for (i = 0; i < BUTTON_COUNT; ++i) {
		wsprintf(temp, "ButtonAction%d", i);
		bRet = bRet && reg.SaveKey(g_pszKey, temp, g_Button_Data[i*4+1]);
	}
	bRet = bRet && reg.SaveKey(g_pszKey,"EnableAtLaunch", g_EnableAtLaunch);
	
	return bRet;
	
}

BOOL LoadSettings()
{
	CRegistry reg;
	char temp[20];
	int i;

	BOOL bRet = reg.LoadKey(g_pszKey,"Speed",(DWORD&)g_Speed);
	bRet = bRet && reg.LoadKey(g_pszKey,"Speed2",(DWORD&)g_Speed2);
	bRet = bRet && reg.LoadKey(g_pszKey,"Pool",(DWORD&)g_Pool);
	bRet = bRet && reg.LoadKey(g_pszKey,"Step",(DWORD&)g_Step);
	for (i = 0; i < BUTTON_COUNT; ++i) {
		wsprintf(temp, "ButtonAction%d", i);
		bRet = bRet && reg.LoadKey(g_pszKey, temp, (DWORD&)g_Button_Data[i*4+1]);
	}
	bRet = bRet && reg.LoadKey(g_pszKey,"EnableAtLaunch", (DWORD&)g_EnableAtLaunch);

	return bRet;
}

BOOL ShowAboutBox()
{		
	return SUCCEEDED(MessageBox(g_hwndMain,"Code: Luca Martinetti, Ian Mahuron\nhttp://www.phatsoft.net/joymouse","About Phatsoft JoyMouse 2.0",MB_ICONINFORMATION | MB_TOPMOST));
}

BOOL ShowOptions()
{
	return SUCCEEDED(DialogBox(g_hInst,MAKEINTRESOURCE(IDD_OPTIONS),g_hwndMain,OptionsDlgProc));
}

DWORD WINAPI UpdateLoop(LPVOID lpNULL)
{
	while(g_bUpdateLoop)
	{
		if(!(GetJoyCapabilities() && RefreshState()))
		{
			if (!g_bGrayed)
			{
				// Change icon to grayed
				g_TrayIcon->ChangeIcon(g_hInst,IDI_MOUSED);
				g_TrayIcon->ChangeTip("Joystick not connected!");
				g_bGrayed = true;
			}
		}
		else
		{
			if (g_bGrayed)
			{
				// Change icon to active
				g_TrayIcon->ChangeIcon(g_hInst,IDI_MOUSE);
				g_TrayIcon->ChangeTip("Phatsoft Joymouse 2.0");
				g_bGrayed = false;
			}
		}
		Sleep(1000/g_Pool);
	}
	return true;
}

BOOL GetJoyCapabilities()
{
	if( joyGetDevCaps(JOYSTICKID1,&g_jc,sizeof(g_jc)) != JOYERR_NOERROR) return false;
	g_Step = (DWORD)(g_jc.wXmax * 0.2);
	return true;
}

void PopulateCombo(int ID, int sel) {
	HWND hWndControl = GetDlgItem(g_hwndOptions,ID);
	_ASSERTE(hWndControl);

	SendMessage(hWndControl,CB_ADDSTRING, 0, (LPARAM)"(none)");
	SendMessage(hWndControl,CB_ADDSTRING, 0, (LPARAM)"Left click");
	SendMessage(hWndControl,CB_ADDSTRING, 0, (LPARAM)"Middle click");
	SendMessage(hWndControl,CB_ADDSTRING, 0, (LPARAM)"Right click");
	SendMessage(hWndControl,CB_ADDSTRING, 0, (LPARAM)"Double click");
	SendMessage(hWndControl,CB_ADDSTRING, 0, (LPARAM)"Accellerate cursor");
	SendMessage(hWndControl,CB_ADDSTRING, 0, (LPARAM)"Enable/Disable joystick");
	SendMessage(hWndControl,CB_SETCURSEL, (WPARAM) sel, 0);
}

void PopulateCombos(void) {
	int i;
	for(i = 0; i < BUTTON_COUNT; ++i) {
		PopulateCombo(g_Button_Data[4*i+2], g_Button_Data[4*i+1]);
	}
}


void ButtonAction(int action, bool pressed) {
	if(g_bEnabled) 
	{ // actions that can be disabled
		switch(action) 
		{ 
			case ACTION_LEFT_CLICK: {
				if(pressed) { // currently pressed
					if (!g_bLeftDown) {
						g_bLeftDown = true;
						mouse_event(MOUSEEVENTF_LEFTDOWN,0,0,0,0);
					}
				} else { // currently released
					if (g_bLeftDown) {
						g_bLeftDown = false;
						mouse_event(MOUSEEVENTF_LEFTUP,0,0,0,0);
					}
				}
			}
			break;
			
			case ACTION_MIDDLE_CLICK: {
				if(pressed) { // currently pressed
					if (!g_bMiddleDown) {
						g_bMiddleDown = true;
						mouse_event(MOUSEEVENTF_MIDDLEDOWN,0,0,0,0);
					}
				} else { // currently released
					if (g_bMiddleDown) {
						g_bMiddleDown = false;
						mouse_event(MOUSEEVENTF_MIDDLEUP,0,0,0,0);
					}
				}
			}
			break;

			case ACTION_RIGHT_CLICK: {
				if(pressed) { // currently pressed
					if (!g_bRightDown) {
						g_bRightDown = true;
						mouse_event(MOUSEEVENTF_RIGHTDOWN,0,0,0,0);
					}
				} else { // currently released
					if (g_bRightDown) {
						g_bRightDown = false;
						mouse_event(MOUSEEVENTF_RIGHTUP,0,0,0,0);
					}
				}
			}
			break;
			
			case ACTION_DOUBLE_CLICK: {
				if(pressed) { // currently pressed
					if (!g_bDoubleDown) {
						g_bDoubleDown = true;
						// simulate double click
						mouse_event(MOUSEEVENTF_LEFTDOWN,0,0,0,0);
						mouse_event(MOUSEEVENTF_LEFTUP,0,0,0,0);
						mouse_event(MOUSEEVENTF_LEFTDOWN,0,0,0,0);
						mouse_event(MOUSEEVENTF_LEFTUP,0,0,0,0);
					}
				} else { // currently released
					if (g_bDoubleDown) {
						g_bDoubleDown = false;
						mouse_event(MOUSEEVENTF_LEFTUP,0,0,0,0);
					}
				}
			}
			break;
			
			case ACTION_ACCEL: {
				if(pressed) { // currently pressed
					if (!g_bAccel) {
						g_bAccel = true;
					}
				} else { // currently released
					if (g_bAccel) {
						g_bAccel = false;
					}
				}
			}
			break;
			
			default:
			break;
		}
	}
	
	switch(action) 
	{ 
		case ACTION_DISABLE: {
			if(pressed)
			{
				if (!g_bEnableButton) 
				{
					g_bEnableButton = true;
				}
			} 
			else 
			{
				if (g_bEnableButton) 
				{
					g_bEnableButton = false;
					Enable(!g_bEnabled); // disable
				}
			}
		}
		break;
		default:
		break;
	}

}

void UnsetAction(int action, int exclude) {
	int i;
	for(i = 0; i < BUTTON_COUNT; i++) {
		if (i == exclude)
			continue;
		if (g_Button_Data[i*4+1] == action) {
			g_Button_Data[i*4+1] = ACTION_NONE;
			SendDlgItemMessage(g_hwndOptions, g_Button_Data[i*4+2], CB_SETCURSEL, ACTION_NONE, 0);
		}
	}
}

void Enable(bool enable) {
	if(enable) {
		g_bEnabled = true;
		if (!g_bGrayed) g_TrayIcon->ChangeIcon(g_hInst, IDI_MOUSE);
		sndPlaySound("enabled.wav", 0);
	} else {
		g_bEnabled = false;
		g_TrayIcon->ChangeIcon(g_hInst, IDI_MOUSED);
		sndPlaySound("disabled.wav", 0);
	}
}

HINSTANCE GotoURL(LPCTSTR url, int showcmd)
{
    TCHAR key[MAX_PATH + MAX_PATH];	

    // First try ShellExecute()
    HINSTANCE result = ShellExecute(NULL, _T("open"), url, NULL,NULL, showcmd);

    // If it failed, get the .htm regkey and lookup the program
    if ((UINT)result <= HINSTANCE_ERROR) 
		{		
		
        if (GetRegKey(HKEY_CLASSES_ROOT, _T(".htm"), key) == ERROR_SUCCESS) 
				{
            lstrcat(key, _T("\\shell\\open\\command"));

            if (GetRegKey(HKEY_CLASSES_ROOT,key,key) == ERROR_SUCCESS) 
						{
                TCHAR *pos;
                pos = _tcsstr(key, _T("\"%1\""));
                if (pos == NULL) 
								{																			 // No quotes found
                    pos = strstr(key, _T("%1"));       // Check for %1, without quotes
                    if (pos == NULL)                   // No parameter at all...
                        pos = key+lstrlen(key)-1;
                    else
                        *pos = '\0';                   // Remove the parameter
                }
                else
                    *pos = '\0';                       // Remove the parameter

                lstrcat(pos, _T(" "));
                lstrcat(pos, url);
                result = (HINSTANCE) WinExec(key,showcmd);
            }
        }
	}
	  
    return result;
}
LONG GetRegKey(HKEY key, LPCTSTR subkey, LPTSTR retdata)
{
    HKEY hkey;
    LONG retval = RegOpenKeyEx(key, subkey, 0, KEY_QUERY_VALUE, &hkey);

    if (retval == ERROR_SUCCESS) {
        long datasize = MAX_PATH;
		TCHAR data[MAX_PATH];
		RegQueryValue(hkey, NULL, data, &datasize);
		lstrcpy(retdata,data);
		RegCloseKey(hkey);
    }

    return retval;
}