/////////////////////////////////////////////////////////////////////////////////////////
// File.......: TRAY.CPP
// Linguaggio.: C++, Win32, MS Visual C++ 6.0
// Descrizione: Implementazione della classe ZTray per la gestione delle Tray Icon
// Autore.....: Lorenzo Novara
// Data.......: 10 gennaio  2001
// Revisione..: 15 gennaio  2001


// Include i prototipi delle funzioni esportate e delle API di Windows
#include "StdAfx.h"
#include "ZTray.h"


ZTray::ZTray(HWND hWnd, UINT uTrayIconID, UINT uCallbackMessage)
{
    m_nid.hWnd = hWnd;
    m_nid.uID  = uTrayIconID;
    m_nid.cbSize = sizeof(NOTIFYICONDATA);
    m_nid.uCallbackMessage = uCallbackMessage;
    m_Active = FALSE;
}


ZTray::~ZTray()
{
    if(m_Active)
        Hide();
}


BOOL ZTray::Show(PSTR pszTip, HICON hIcon)
{    
    BOOL  res;

    // Verifica che l'icona non sia gi visibile
    if(m_Active!=FALSE)
        return FALSE;
    
    // Verifica i parametri
    if(hIcon==FALSE)
        return FALSE;

    // Riempie i campi della struttura NOTIFYICONDATA    
    m_nid.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP;    
    m_nid.hIcon  = hIcon;
    
    // Se  passata una stringa la scrive...
    if (pszTip) {
        if (::lstrlen(pszTip)>63) // (il buffer ha capienza 64)
            return FALSE;
        ::lstrcpyn(m_nid.szTip, pszTip, sizeof(m_nid.szTip)); 
    } else //...altrimenti ne scrive una vuota
        m_nid.szTip[0] = '\0';

    // Registra l'icona
    res = ::Shell_NotifyIcon(NIM_ADD, &m_nid);

    if(res) {
        m_Active = TRUE;
        return TRUE;
    } else
        return FALSE;
}


BOOL ZTray::Show(PSTR pszTip, HINSTANCE hInstance, PSTR pszResourceName)
{    
    BOOL  res;
    
    // Verifica i parametri
    if(pszResourceName==NULL)
        return FALSE;
    
    // Carica l'icona
    HICON hIcon=::LoadIcon(hInstance, pszResourceName);
    if(hIcon==FALSE)
        return FALSE;
    
    // Chiama la versione base del metodo
    res=Show(pszTip, hIcon);
    
    // Distrugge l'icona
    ::DestroyIcon(hIcon);

    return res;
}


BOOL ZTray::Show(PSTR pszTip, HINSTANCE hInstance, UINT nIDResource)
{
    return Show(pszTip, hInstance, MAKEINTRESOURCE(nIDResource));
}


BOOL ZTray::ChangeTip(PSTR pszTip)
{
    BOOL res;
    
    // Verifica che l'icona sia gi visibile
    if(m_Active==FALSE)
        return FALSE;
    
    // Riempie i campi della struttura NOTIFYICONDATA
    m_nid.uFlags = NIF_TIP;
    
    // Se  passata una stringa la scrive...
    if (pszTip) {
        if (::lstrlen(pszTip)>63) // (il buffer ha capienza 64)
            return FALSE;
        ::lstrcpyn(m_nid.szTip, pszTip, sizeof(m_nid.szTip)); 
    } else //...altrimenti ne scrive una vuota
        m_nid.szTip[0] = '\0';

    // Registra l'icona
    res = ::Shell_NotifyIcon(NIM_MODIFY, &m_nid);
    
    if(res) {
        return TRUE;
    } else
        return FALSE;
}


BOOL ZTray::ChangeIcon(HICON hIcon)
{   
    BOOL res;

    // Verifica che l'icona sia gi visibile
    if(m_Active==FALSE)
        return FALSE;
    
    // Verifica i parametri
    if(hIcon==FALSE)
        return FALSE;
    
    // Riempie i campi della struttura NOTIFYICONDATA    
    m_nid.uFlags = NIF_ICON;       
    m_nid.hIcon  = hIcon;

    // Registra l'icona
    res = ::Shell_NotifyIcon(NIM_MODIFY, &m_nid);

    if(res)
        return TRUE;
    else
        return FALSE;
}


BOOL ZTray::ChangeIcon(HINSTANCE hInstance, PSTR pszResourceName)
{   
    BOOL res;    

    // Verifica i parametri
    if(pszResourceName==NULL)
        return FALSE;
    
    // Carica l'icona
    HICON hIcon=(HICON) ::LoadIcon(hInstance, pszResourceName);//::LoadImage(hInstance, pszResourceName, IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
    if(hIcon==FALSE)
        return FALSE;
    
    // Chiama la versione base del metodo
    res=ChangeIcon(hIcon);
    
    // Distrugge l'icona
    ::DestroyIcon(hIcon);

    return res;
}


BOOL ZTray::ChangeIcon(HINSTANCE hInstance, UINT nIDResource)
{
    return ChangeIcon(hInstance, MAKEINTRESOURCE(nIDResource));
}


BOOL ZTray::Hide()
{
    BOOL res;     

    // Verifica che l'icona sia visibile
    if(m_Active==FALSE)
        return FALSE;
    
    //Rimuove l'icona
    res = ::Shell_NotifyIcon(NIM_DELETE, &m_nid);
    
    m_Active=FALSE;

    if(res)
        return TRUE;
    else
        return FALSE;
}
